/* Copyright 2014-2017 Research Foundation State University of New York   */

/* This file is part of QUB Online.                                       */

/* QUB Online is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License as published by   */
/* the Free Software Foundation, either version 3 of the License, or      */
/* (at your option) any later version.                                    */

/* QUB Online is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License,     */
/* named LICENSE.txt, in the QUB Online program directory.  If not, see  */
/* <http://www.gnu.org/licenses/>.                                        */

#include <iostream>
#include <string.h>
using std::cerr;
using std::size_t;

#include "asmffi.h"

extern "C" double* ASMFFI_API dalloc(size_t n) {
  return n ? new double[n] : 0;
}

extern "C" void ASMFFI_API dfree(double *buf) {
  delete [] buf;
}

extern "C" double ASMFFI_API dget(double *buf, size_t i) {
  return buf[i];
}

extern "C" void ASMFFI_API dset(double *buf, size_t i, double x) {
  buf[i] = x;
}

extern "C" float* ASMFFI_API falloc(size_t n) {
  return n ? new float[n] : 0;
}

extern "C" void ASMFFI_API ffree(float *buf) {
  delete [] buf;
}

extern "C" float ASMFFI_API fget(float *buf, size_t i) {
  return buf[i];
}

extern "C" void ASMFFI_API fset(float *buf, size_t i, float x) {
  buf[i] = x;
}

extern "C" int* ASMFFI_API ialloc(size_t n) {
  return n ? new int[n] : 0;
}

extern "C" void ASMFFI_API ifree(int *buf) {
  delete [] buf;
}

extern "C" int ASMFFI_API iget(int *buf, size_t i) {
  return buf[i];
}

extern "C" void ASMFFI_API iset(int *buf, size_t i, int x) {
  buf[i] = x;
}

extern "C" long long ASMFFI_API i64get(short *buf, size_t i) {
  return buf[i];
}

extern "C" void ASMFFI_API i64set(long long *buf, size_t i, long long x) {
  buf[i] = x;
}

extern "C" short ASMFFI_API i16get(short *buf, size_t i) {
  return buf[i];
}

extern "C" void ASMFFI_API i16set(short *buf, size_t i, short x) {
  buf[i] = x;
}

extern "C" char ASMFFI_API i8get(char *buf, size_t i) {
  return buf[i];
}

extern "C" void ASMFFI_API i8set(char *buf, size_t i, char x) {
  buf[i] = x;
}

extern "C" void** ASMFFI_API palloc(size_t n) {
  return n ? new void*[n] : 0;
}

extern "C" void ASMFFI_API pfree(void **buf) {
  delete [] buf;
}

extern "C" void* ASMFFI_API pget(void **buf, size_t i) {
  return buf[i];
}

extern "C" void ASMFFI_API pset(void **buf, size_t i, void *x) {
  buf[i] = x;
}

extern "C" int** ASMFFI_API ialloc2(size_t n, size_t m) {
  if ( (!n) || (!m) )
    return 0;
  int *buf = new int[n*m];
  int **bufs = new int*[n];
  for (size_t i=0; i<n; ++i) {
    bufs[i] = buf + i*m;
  }
  return bufs;
}

extern "C" void ASMFFI_API ifree2(int **bufs) {
  if ( bufs ) {
    delete [] bufs[0];
    delete [] bufs;
  }
}

extern "C" float** ASMFFI_API falloc2(size_t n, size_t m) {
  if ( (!n) || (!m) )
    return 0;
  float *buf = new float[n*m];
  float **bufs = new float*[n];
  for (size_t i=0; i<n; ++i) {
    bufs[i] = buf + i*m;
  }
  return bufs;
}

extern "C" void ASMFFI_API ffree2(float **bufs) {
  if ( bufs ) {
    delete [] bufs[0];
    delete [] bufs;
  }
}

extern "C" float* ASMFFI_API frow2(float **bufs, size_t i) {
  return bufs[i];
}

extern "C" double** ASMFFI_API dalloc2(size_t n, size_t m) {
  if ( (!n) || (!m) )
    return 0;
  double *buf = new double[n*m];
  double **bufs = new double*[n];
  for (size_t i=0; i<n; ++i) {
    bufs[i] = buf + i*m;
  }
  return bufs;
}

extern "C" void ASMFFI_API dfree2(double **bufs) {
  if ( bufs ) {
    delete [] bufs[0];
    delete [] bufs;
  }
}

extern "C" double* ASMFFI_API drow2(double **bufs, size_t i) {
  return bufs[i];
}

extern "C" void ASMFFI_API wrap_memcpy(void *dest, void *src, size_t bytes) {
  memcpy(dest, src, bytes);
}

extern "C" int ASMFFI_API consoleLog(const char *msg, void *data) {
#ifdef EMSCRIPTEN
  EM_ASM_({
      console.log(Pointer_stringify($0));
    }, msg);
  return 0;
#else
  std::cerr << msg;
#endif
  return 0;
}

#ifdef EMSCRIPTEN
callbk_reportstream console(consoleLog, 0);
#endif
